/*
 *  FileDocumentSpaceStorage.java - SE@SA Lab 2006
 *  
 *  This file is part of COCONUT.
 *
 *  COCONUT is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  COCONUT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with COCONUT; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.sesalab.IR.documentSpace.storage;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Map;

import org.sesalab.IR.matrix.ISparseMatrix;

/**
 * Classe che implementa la persistenza dello spazio dei documenti mediante l'uso di file di testo
 * 
 * @author Paola Sgueglia
 * @param <DocumentType> 
 * @param <TermType> 
 */
public class FileDocumentSpaceStorage<DocumentType, TermType> implements IDocumentSpaceStorage<DocumentType, TermType> {
	/**
	 * Nome del file dei termini della collezione
	 */
	public static final String TERM_FILE_NAME = "terms.dat";
	
	/**
	 * Nome del file dei documenti della collezione
	 */
	public static final String DOCUMENT_FILE_NAME = "documents.dat";

	/**
	 * Nome del file della matrice termini-documenti
	 */
	public static final String MATRIX_FILE_NAME = "matrix.dat";
	
	// Path per la memorizzazione dello spazio dei documenti
	private String documentSpacePath;
	
	/**
	 * Crea una nuova istanza della classe
	 * @param pDocumentSpacePath Il path per la memorizzazione dello spazio dei documenti
	 * @throws IOException 
	 * @throws FileNotFoundException 
	 */
	public FileDocumentSpaceStorage(String pDocumentSpacePath) throws FileNotFoundException, IOException {
		this.documentSpacePath = pDocumentSpacePath;
		File directory = new File(this.documentSpacePath);
		if (!directory.exists())
			directory.mkdirs();
	}
	
	/**
	 * @see org.sesalab.IR.documentSpace.storage.IDocumentSpaceStorage#loadTerms()
	 */
	@SuppressWarnings("unchecked")
	public Map<Integer, TermType> loadTerms() throws IOException, ClassNotFoundException {
		String termFileName = this.documentSpacePath + FileDocumentSpaceStorage.TERM_FILE_NAME;  
		ObjectInputStream termInputStream = new ObjectInputStream(new FileInputStream(termFileName));
		
		Map<Integer, TermType> loadedTerms = (Map<Integer, TermType>) termInputStream.readObject();
		termInputStream.close();
		
		return loadedTerms;
	}
	
	/**
	 * @see org.sesalab.IR.documentSpace.storage.IDocumentSpaceStorage#saveTerms(java.util.Map)
	 */
	public void saveTerms(Map<Integer, TermType> pTerms) throws IOException {
		String termFileName = this.documentSpacePath + FileDocumentSpaceStorage.TERM_FILE_NAME;  
		ObjectOutputStream termOutputStream = new ObjectOutputStream(new FileOutputStream(termFileName));
		
		termOutputStream.writeObject(pTerms);
		termOutputStream.close();
	}

	/**
	 * @see org.sesalab.IR.documentSpace.storage.IDocumentSpaceStorage#loadDocuments()
	 */
	@SuppressWarnings("unchecked")
	public Map<Integer, DocumentType> loadDocuments() throws IOException, ClassNotFoundException {
		String documentFileName = this.documentSpacePath + FileDocumentSpaceStorage.DOCUMENT_FILE_NAME;  
		ObjectInputStream documentInputStream = new ObjectInputStream(new FileInputStream(documentFileName));
		
		Map<Integer, DocumentType> loadedDocuments = (Map<Integer, DocumentType>) documentInputStream.readObject();
		documentInputStream.close();
		
		return loadedDocuments;
	}

	/**
	 * @see org.sesalab.IR.documentSpace.storage.IDocumentSpaceStorage#saveDocuments(java.util.Map)
	 */
	public void saveDocuments(Map pDocuments) throws IOException {
		String documentFileName = this.documentSpacePath + FileDocumentSpaceStorage.DOCUMENT_FILE_NAME;  
		ObjectOutputStream documentOutputStream = new ObjectOutputStream(new FileOutputStream(documentFileName));
		
		documentOutputStream.writeObject(pDocuments);
		documentOutputStream.close();
	}

	/**
	 * @see org.sesalab.IR.documentSpace.storage.IDocumentSpaceStorage#loadMatrix()
	 */
	public ISparseMatrix loadMatrix() throws IOException, ClassNotFoundException {
		String matrixFileName = this.documentSpacePath + FileDocumentSpaceStorage.MATRIX_FILE_NAME;  
		ObjectInputStream matrixInputStream = new ObjectInputStream(new FileInputStream(matrixFileName));
		
		ISparseMatrix loadedMatrix = (ISparseMatrix) matrixInputStream.readObject();
		matrixInputStream.close();
		
		return loadedMatrix;
	}

	/**
	 * @see org.sesalab.IR.documentSpace.storage.IDocumentSpaceStorage#saveMatrix(org.sesalab.IR.matrix.ISparseMatrix)
	 */
	public void saveMatrix(ISparseMatrix pMatrix) throws IOException {
		String matrixFileName = this.documentSpacePath + FileDocumentSpaceStorage.MATRIX_FILE_NAME;  
		ObjectOutputStream matrixOutputStream = new ObjectOutputStream(new FileOutputStream(matrixFileName));
		
		matrixOutputStream.writeObject(pMatrix);
		matrixOutputStream.close();
	}
}