/*
 *  IDocumentSpaceStorage.java - SE@SA Lab 2006
 *  
 *  This file is part of COCONUT.
 *
 *  COCONUT is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  COCONUT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with COCONUT; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.sesalab.IR.documentSpace.storage;

import java.io.IOException;
import java.util.Map;

import org.sesalab.IR.matrix.ISparseMatrix;

/**
 * Definisce l'interfaccia per la persistenza dello spazio dei documenti
 * rappresentato dalla matrice termini-documenti
 * 
 * @author Paola Sgueglia
 * @param <DocumentType>
 * @param <TermType>
 */
public interface IDocumentSpaceStorage<DocumentType, TermType> {
	/**
	 * Consente di caricare i termini della collezione
	 * 
	 * @return Una mappa dei termini della collezione (key = riga - valore =
	 *         termine)
	 * @throws ClassNotFoundException
	 *         Generata quando non  possibile convertire l'oggetto caricato al
	 *         tipo Map
	 * @throws IOException
	 *         Generata quando non  possibile accedere al filesystem
	 */
	public Map<Integer, TermType> loadTerms() throws IOException, ClassNotFoundException;

	/**
	 * Consente di salavre i termini della collezione
	 * 
	 * @param pTerms
	 *        I termini della collezione (key = riga - valore = termine)
	 * @throws IOException
	 *         Generata quando non  possibile accedere al filesystem
	 */
	public void saveTerms(Map<Integer, TermType> pTerms) throws IOException;

	/**
	 * Consente di caricare i documenti della collezione
	 * 
	 * @return Una mappa dei documenti della collezione (key = colonna - valore =
	 *         documento)
	 * @throws ClassNotFoundException
	 *         Generata quando non  possibile convertire l'oggetto caricato al
	 *         tipo Map
	 * @throws IOException
	 *         Generata quando non  possibile accedere al filesystem
	 */
	public Map<Integer, DocumentType> loadDocuments() throws IOException, ClassNotFoundException;

	/**
	 * Consente di salvare i documenti della collezione
	 * 
	 * @param pDocuments
	 *        I documenti della collezione (key = riga - valore = documento)
	 * @throws IOException
	 *         Generata quando non  possibile accedere al filesystem
	 */
	public void saveDocuments(Map<Integer, DocumentType> pDocuments) throws IOException;

	/**
	 * Consente di caricare la matrice termini-documenti
	 * 
	 * @return La matrice sparsa termini-documenti
	 * @throws ClassNotFoundException
	 *         Generata quando non  possibile convertire l'oggetto caricato al
	 *         tipo ISparseMatrix
	 * @throws IOException
	 *         Generata quando non  possibile accedere al filesystem
	 */
	public ISparseMatrix loadMatrix() throws IOException, ClassNotFoundException;

	/**
	 * Consente di salvare la matrice termini-documenti
	 * 
	 * @param pMatrix
	 *        La matrice termini-documenti
	 * @throws IOException
	 *         Generata quando non  possibile accedere al filesystem
	 */
	public void saveMatrix(ISparseMatrix pMatrix) throws IOException;
}
