/*
 *  XMLDocumentSpaceStorage.java - SE@SA Lab 2006
 *  
 *  This file is part of COCONUT.
 *
 *  COCONUT is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  COCONUT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with COCONUT; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.sesalab.IR.documentSpace.storage;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Map;

import org.sesalab.IR.matrix.ISparseMatrix;

/**
 * Classe che implementa la persistenza dello spazio dei documenti mediante l'uso di file XML
 * 
 * @author Paola Sgueglia
 * @param <DocumentType> 
 * @param <TermType> 
 */
public class XMLDocumentSpaceStorage<DocumentType, TermType> implements IDocumentSpaceStorage<DocumentType, TermType> {
	/**
	 * Nome del file dei termini della collezione
	 */
	public static final String TERM_FILE_NAME = "terms.xml";
	
	/**
	 * Nome del file dei documenti della collezione
	 */
	public static final String DOCUMENT_FILE_NAME = "documents.xml";

	/**
	 * Nome del file della matrice termini-documenti
	 */
	public static final String MATRIX_FILE_NAME = "matrix.xml";
	
	// Path per la memorizzazione dello spazio dei documenti
	private String documentSpacePath;
	
	/**
	 * Crea una nuova istanza della classe
	 * @param pDocumentSpacePath Il path per la memorizzazione dello spazio dei documenti
	 * @throws IOException 
	 * @throws FileNotFoundException 
	 */
	public XMLDocumentSpaceStorage(String pDocumentSpacePath) throws FileNotFoundException, IOException {
		this.documentSpacePath = pDocumentSpacePath;
		File directory = new File(this.documentSpacePath);
		if (!directory.exists())
			directory.mkdirs();
	}
	
	/**
	 * @see org.sesalab.IR.documentSpace.storage.IDocumentSpaceStorage#loadTerms()
	 */
	@SuppressWarnings("unchecked")
	public Map<Integer, TermType> loadTerms() throws IOException, ClassNotFoundException {
		String termsFileName = this.documentSpacePath + XMLDocumentSpaceStorage.TERM_FILE_NAME;
		XMLDecoder xmlDecoder = new XMLDecoder(new BufferedInputStream(new FileInputStream(termsFileName)));
		Map<Integer, TermType> loadedTerms = (Map<Integer, TermType>) xmlDecoder.readObject();
		xmlDecoder.close(); 
		
		return loadedTerms;
	}
	
	/**
	 * @see org.sesalab.IR.documentSpace.storage.IDocumentSpaceStorage#saveTerms(java.util.Map)
	 */
	public void saveTerms(Map<Integer, TermType> pTerms) throws IOException {
		String termsFileName = this.documentSpacePath + XMLDocumentSpaceStorage.TERM_FILE_NAME;
		XMLEncoder xmlEncoder = new XMLEncoder(new BufferedOutputStream(new FileOutputStream(termsFileName)));
		xmlEncoder.writeObject(pTerms);
		xmlEncoder.close(); 
	}

	/**
	 * @see org.sesalab.IR.documentSpace.storage.IDocumentSpaceStorage#loadDocuments()
	 */
	@SuppressWarnings("unchecked")
	public Map<Integer, DocumentType> loadDocuments() throws IOException {
		String documentsFileName = this.documentSpacePath + XMLDocumentSpaceStorage.DOCUMENT_FILE_NAME;
		XMLDecoder xmlDecoder = new XMLDecoder(new BufferedInputStream(new FileInputStream(documentsFileName)));
		Map<Integer, DocumentType> loadedDocuments = (Map<Integer, DocumentType>) xmlDecoder.readObject();
		xmlDecoder.close(); 
		
		return loadedDocuments;
	}

	/**
	 * @see org.sesalab.IR.documentSpace.storage.IDocumentSpaceStorage#saveDocuments(java.util.Map)
	 */
	public void saveDocuments(Map<Integer, DocumentType> pDocuments) throws IOException {
		String documentsFileName = this.documentSpacePath + XMLDocumentSpaceStorage.DOCUMENT_FILE_NAME;
		XMLEncoder xmlEncoder = new XMLEncoder(new BufferedOutputStream(new FileOutputStream(documentsFileName)));
		xmlEncoder.writeObject(pDocuments);
		xmlEncoder.close(); 
	}

	/**
	 * @see org.sesalab.IR.documentSpace.storage.IDocumentSpaceStorage#loadMatrix()
	 */
	public ISparseMatrix loadMatrix() throws IOException, ClassNotFoundException {
		String matrixFileName = this.documentSpacePath + XMLDocumentSpaceStorage.MATRIX_FILE_NAME;
		XMLDecoder xmlDecoder = new XMLDecoder(new BufferedInputStream(new FileInputStream(matrixFileName)));
		ISparseMatrix loadedMatrix = (ISparseMatrix) xmlDecoder.readObject();
		xmlDecoder.close(); 
		
		return loadedMatrix;
	}

	/**
	 * @see org.sesalab.IR.documentSpace.storage.IDocumentSpaceStorage#saveMatrix(org.sesalab.IR.matrix.ISparseMatrix)
	 */
	public void saveMatrix(ISparseMatrix pMatrix) throws IOException {
		String matrixFileName = this.documentSpacePath + XMLDocumentSpaceStorage.MATRIX_FILE_NAME;
		XMLEncoder xmlEncoder = new XMLEncoder(new BufferedOutputStream(new FileOutputStream(matrixFileName)));
		xmlEncoder.writeObject(pMatrix);
		xmlEncoder.close(); 
	}
}