/*
 *  NormalisedTfIdfWeigher.java - SE@SA Lab 2006
 *  
 *  This file is part of COCONUT.
 *
 *  COCONUT is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  COCONUT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with COCONUT; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.sesalab.IR.documentSpace.weigher;

import org.sesalab.IR.matrix.CSCSparseMatrix;
import org.sesalab.IR.matrix.ISparseMatrix;
import org.sesalab.IR.matrix.exception.InvalidIndexException;

/**
 * Classe che implementa l'algoritmo di pesatura della matrice termini-documenti
 * denominato Normalised TfIdf
 * 
 * @author Paola Sgueglia
 */
public class NormalisedTfIdfWeigher implements IWeigherSparseMatrix {

	/**
	 * Crea una nuova istanza della classe
	 */
	public NormalisedTfIdfWeigher() {
		super();
	}

	/**
	 * @see org.sesalab.IR.documentSpace.weigher.IWeigherSparseMatrix#weigh(org.sesalab.IR.matrix.ISparseMatrix)
	 */
	public ISparseMatrix weigh(ISparseMatrix pSparseMatrix) {
		// Variabili di appoggio
		int index;
		double normalisedTermFrequency, inverseDocumentFrequency;

		// Dichiarazione dei vettore necessari per il calcolo dei pesi
		double[] documentFrequency;

		// Inizializzazione
		int rows[] = pSparseMatrix.getNotNullRowIndices();
		int columns[] = pSparseMatrix.getNotNullColumnIndices();
		double values[] = pSparseMatrix.getNotNullValues();
		double weightedValues[] = new double[values.length];

		// Inizializzazione del vettore contenente la frequenza dei termini
		documentFrequency = new double[pSparseMatrix.getRowCount()];
		// Calcolo della frequenza dei documenti
		for (index = 0; index < rows.length; index++) {
			documentFrequency[rows[index]-1]++;
		}
		
		double maxFrequency = values[0];
		for (int i = 1; i < values.length; i++) {
			if (values[i] > maxFrequency)
				maxFrequency = values[i];
		}

		// Definizione del peso dei termini
		for (index = 0; index < values.length; index++) {
			normalisedTermFrequency = values[index] / maxFrequency;
			inverseDocumentFrequency = Math.log((pSparseMatrix.getColumnCount() / documentFrequency[rows[index] - 1]) + 1);
			weightedValues[index] = normalisedTermFrequency * inverseDocumentFrequency;
		}

		// Creazione matrice sparsa
		ISparseMatrix weightedMatrix = new CSCSparseMatrix();
		((CSCSparseMatrix) weightedMatrix).init(pSparseMatrix.getRowCount(), pSparseMatrix.getColumnCount());
		for (index = 0; index < values.length; index++) {
			try {
				weightedMatrix.set(rows[index], columns[index], weightedValues[index]);
			} catch (InvalidIndexException e) {
				// Eccezione che non si pu mai verificare
			}
		}
		return weightedMatrix;
	}
}
