/*
 *  ISparseMatrix.java - SE@SA Lab 2006
 *  
 *  This file is part of COCONUT.
 *
 *  COCONUT is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  COCONUT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with COCONUT; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.sesalab.IR.matrix;

/**
 * Definisce l'interfaccia per una matrice sparsa
 * 
 * @author Paola Sgueglia
 */
public interface ISparseMatrix extends IMatrix {

	/**
	 * Restituisce gli indici delle colonne non nulle
	 * 
	 * @return Gli indici delle colonne non nulle
	 */
	public int[] getNotNullColumnIndices();

	/**
	 * Restituisce gli indici delle righe non nulle
	 * 
	 * @return Gli indici delle righe non nulle
	 */
	public int[] getNotNullRowIndices();

	/**
	 * Restituisce tutti i valori non nulli
	 * 
	 * @return I valori non nulli
	 */
	public double[] getNotNullValues();

	/**
	 * Verifica se la colonna <code>pColumn</code>  completamente nulla
	 * 
	 * @param pColumn
	 *        La colonna da verificare
	 * @return <code>true</code> se la colonna  nulla, <code>false</code>
	 *         altrimenti
	 */
	public boolean isNullColumn(int pColumn);

	/**
	 * Verifica se la riga <code>pRow</code>  completamente nulla
	 * 
	 * @param pRow
	 *        La riga da verificare
	 * @return <code>true</code> se la riga  nulla, <code>false</code>
	 *         altrimenti
	 */
	public boolean isNullRow(int pRow);
}