/*
 *  Feedbacks.java - SE@SA Lab 2006
 *  
 *  This file is part of COCONUT.
 *
 *  COCONUT is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  COCONUT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with COCONUT; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.sesalab.IR.model;

import java.util.LinkedList;
import java.util.List;

/**
 * Classe che modella i feedback fornite dall'utente per una query
 * 
 * @param <DocumentType>
 *        Il tipo di documento della collezione
 * @author Paola Sgueglia
 */
public class Feedbacks<DocumentType> {
	
	/**
	 * Crea una nuova istanza
	 */
	public Feedbacks() {
		this.correctDocuments = new LinkedList<DocumentType>();
		this.falsePositiveDocuments = new LinkedList<DocumentType>();
	}

	/**
	 * @return L'elenco dei documenti effettivamente rilevanti per la query
	 */
	public List<DocumentType> getCorrectDocuments() {
		return this.correctDocuments;
	}

	/**
	 * @return L'elenco dei documenti recuperati e non rilevanti per la query
	 */
	public List<DocumentType> getFalsePositiveDocuments() {
		return this.falsePositiveDocuments;
	}

	/**
	 * Setta l'elenco dei documenti recuperati ed effettivamente rilevanti per
	 * la query
	 * 
	 * @param pCorrectDocuments
	 *        L'elenco dei documenti recuperati ed effettivamente rilevanti per
	 *        la query
	 */
	public void setCorrectDocuments(List<DocumentType> pCorrectDocuments) {
		this.correctDocuments = pCorrectDocuments;
	}

	/**
	 * Setta l'elenco dei documenti recuperati e non rilevanti per la query
	 * 
	 * @param pFalsePositiveDocuments
	 *        L'elenco dei documenti recuperati e non rilevanti per la query
	 */
	public void setFalsePositiveDocuments(List<DocumentType> pFalsePositiveDocuments) {
		this.falsePositiveDocuments = pFalsePositiveDocuments;
	}

	// L'elenco dei documenti recuperati ed effettivamente rilevanti per la
	// query
	private List<DocumentType> correctDocuments;

	// L'elenco dei documenti recuperati e non rilevanti per la query
	private List<DocumentType> falsePositiveDocuments;

	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		String str = "Correct documents: " + this.correctDocuments + " - "; 
		str += "False positives: " + this.falsePositiveDocuments; 
		return  str;
	}
}