/*
 *  SSVDManager.java - SE@SA Lab 2006
 *  
 *  This file is part of COCONUT.
 *
 *  COCONUT is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  COCONUT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with COCONUT; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.sesalab.IR.model.LSI.SSVD;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.sesalab.IR.matrix.ISparseMatrix;
import org.sesalab.IR.model.LSI.LatentSemanticIndexing;

import Jama.Matrix;
import Jama.SingularValueDecomposition;

/**
 * Classe utilizzata per gestire la decomposizione ai valori singolari
 * 
 * @author Paola Sgueglia
 */
public class SSVDManager {
	/**
	 * Costrisce una nuova istanza della classe
	 * 
	 * @param pSparseMatrix
	 *        La matrice sparsa da decomporre
	 * @param pWorkingFolder
	 *        La directory di lavoro
	 */
	public SSVDManager(ISparseMatrix pSparseMatrix, String pWorkingFolder) {
		this.matrix = new Matrix(pSparseMatrix.toArray());
		this.workingFolder = pWorkingFolder;
		try {
			this.singularValueDecomposition = this.load();
		} catch (Exception e) {
			this.singularValueDecomposition = null;
		}
	}

	/**
	 * @param pForceRebuilding
	 *        Consente di forzare il rebuilding della decomposizione ai valori
	 *        singolari
	 * @return La decomposizione ai valori singolari
	 */
	public SingularValueDecomposition getSingularValueDecomposition(boolean pForceRebuilding) {
		if ((!pForceRebuilding) && (this.singularValueDecomposition != null))
			return (this.singularValueDecomposition);
		SingularValueDecomposition newSingularValueDecomposition = new SingularValueDecomposition(this.matrix);
		this.singularValueDecomposition = newSingularValueDecomposition;
		try {
			this.save(newSingularValueDecomposition);
		} catch (Exception exception) {
			exception.printStackTrace();
		}
		return newSingularValueDecomposition;
	}

	/**
	 * Consente di settare la matrice da decomporre
	 * 
	 * @param pMatrix
	 *        La matrice da decomporre
	 */
	public void setMatrix(ISparseMatrix pMatrix) {
		this.matrix = new Matrix(pMatrix.toArray());
	}

	/*
	 * @return La decomposizione ai valori singolari caricata
	 * @throws ClassNotFoundException
	 *         Generata quando non  possibile convertire l'oggetto caricato al
	 *         tipo ISparseMatrix
	 * @throws IOException
	 *         Generata quando non  possibile accedere al filesystem
	 */
	private SingularValueDecomposition load() throws IOException, ClassNotFoundException {
		String decompositionFileName = this.workingFolder + "/" + LatentSemanticIndexing.DECOMPOSITION_FILE_NAME;
		ObjectInputStream decompositionInputStream = new ObjectInputStream(new FileInputStream(decompositionFileName));

		SingularValueDecomposition loadedSVD = (SingularValueDecomposition) decompositionInputStream.readObject();
		decompositionInputStream.close();

		return loadedSVD;
	}

	/*
	 * Consente di salvare la decomposizione ai valori singolari
	 * 
	 * @param pSingularValueDecomposition
	 *        La decomposizione ai valori singolari da salvare
	 * @throws IOException
	 *         Generata quando non  possibile accedere al filesystem
	 */
	private void save(SingularValueDecomposition pSingularValueDecomposition) throws IOException {
		String decompositionFileName = this.workingFolder + "/" + LatentSemanticIndexing.DECOMPOSITION_FILE_NAME;
		ObjectOutputStream decompositionOutputStream = new ObjectOutputStream(new FileOutputStream(decompositionFileName));

		decompositionOutputStream.writeObject(pSingularValueDecomposition);
		decompositionOutputStream.close();
	}

	// La matrice da decomporre
	private Matrix matrix;

	// La decomposizione ai valori singolari
	private SingularValueDecomposition singularValueDecomposition;

	// La directory di lavoro
	private String workingFolder;
}