/*
 *  IVectorFeedbackPerformer.java - SE@SA Lab 2006
 *  
 *  This file is part of COCONUT.
 *
 *  COCONUT is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  COCONUT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with COCONUT; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.sesalab.IR.model.vectorFeedbackPerformer;

import java.util.Map;

import org.sesalab.IR.documentSpace.ITermByDocumentMatrix;
import org.sesalab.IR.matrix.IMatrix;
import org.sesalab.IR.matrix.ISparseMatrix;
import org.sesalab.IR.model.Feedbacks;
import org.sesalab.IR.model.exception.IncorrectVectorSpaceException;

/**
 * Definisce l'interfaccia di un classe addetta all'applicazione dei feedback
 * forniti dall'utente. I documenti sono rappresentati da vettori.
 * 
 * @param <DocumentType>
 *        Il tipo che rappresenta i documenti della collezione
 * @param <TermType>
 *        Il tipo che rappresenta i termini della collezione
 * @author Paola Sgueglia
 */
public interface IVectorFeedbackPerformer<DocumentType, TermType> {

	/**
	 * Applica i feedback forniti dall'utente alla matrice termini documenti
	 * 
	 * @param pMatrix
	 *        La matrice termini-documenti
	 * @param pUserFeedbacks
	 *        I feedback forniti dall'utente per ciascuna query
	 * @return La matrice termini-documenti dopo l'applicazione dei feedback
	 * @throws IncorrectVectorSpaceException
	 *         Generata quando i vettori appartengono a spazi diversi
	 */
	public ISparseMatrix apply(ITermByDocumentMatrix<DocumentType, TermType> pMatrix, Map<DocumentType, Feedbacks<DocumentType>> pUserFeedbacks) throws IncorrectVectorSpaceException;

	/**
	 * Applica i feedback forniti dall'utente ad una matrice sparsa che
	 * identifica una matrice termini documenti
	 * 
	 * @param pMatrix
	 *        La matrice sparse
	 * @param pUserFeedbacks
	 *        I feedback forniti dall'utente per ciascuna query (i documenti
	 *        sono rappresentati dalle colonne occupate nella matrice).
	 * @throws IncorrectVectorSpaceException
	 *         Generata quando i vettori appartengono a spazi diversi
	 */
	public void apply(IMatrix pMatrix, Map<Integer, Feedbacks<Integer>> pUserFeedbacks) throws IncorrectVectorSpaceException;
}
