/*
 *  CosineSimilarity.java - SE@SA Lab 2006
 *  
 *  This file is part of COCONUT.
 *
 *  COCONUT is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  COCONUT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with COCONUT; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.sesalab.IR.model.vectorSimilarityPerformer;

import org.sesalab.IR.model.exception.IncorrectVectorSpaceException;
import org.sesalab.IR.model.util.AdvanceMath;

/**
 * @author Paola Sgueglia
 */
public class CosineSimilarity implements IVectorSimilarityPerformer {

	/**
	 * Numero di cifre decimali consentite per il valore si somiglianza
	 */
	public static final int DEFAULT_FRANCTIONAL_DIGITS = 3;

	/**
	 * Crea una nuova istanza della classe
	 */
	public CosineSimilarity() {
		this.maximumFractionDigits = CosineSimilarity.DEFAULT_FRANCTIONAL_DIGITS;
	}

	/**
	 * Crea una nuova istanza della classe
	 * 
	 * @param pMaximumFractionDigits
	 *        Il numero di cifre decimali consentite per il valore si
	 *        somiglianza.
	 */
	public CosineSimilarity(int pMaximumFractionDigits) {
		this.maximumFractionDigits = pMaximumFractionDigits;
	}

	/**
	 * @throws IncorrectVectorSpaceException
	 * @see org.sesalab.IR.model.vectorSimilarityPerformer.IVectorSimilarityPerformer#calculateSimilarity(double[],
	 *      double[])
	 */
	public double calculateSimilarity(double[] source, double[] target) throws IncorrectVectorSpaceException {
		return AdvanceMath.approx(AdvanceMath.calculateCosine(source, target), this.maximumFractionDigits);
	}

	private int maximumFractionDigits;
}
