/*
 *  BasicRankedList.java - SE@SA Lab 2006
 *  
 *  This file is part of COCONUT.
 *
 *  COCONUT is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  COCONUT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with COCONUT; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.sesalab.IR.rankedList;

import java.util.Collection;
import java.util.Iterator;
import java.util.TreeSet;

import org.sesalab.IR.rankedList.cutter.ConstantCutter;
import org.sesalab.IR.rankedList.cutter.ICutter;

/**
 * Fornisce l'implementazione di default della ranked list. La ranked list 
 * rappresentata da un <code>TreeSet</code>
 * 
 * @author Paola Sgueglia
 */
public class BasicRankedList implements IRankedList {

	/**
	 * Costruisce una ranked list vuota
	 */
	public BasicRankedList() {
		this.rankedList = new TreeSet<RankedListElement>();
		this.maxSimilarityValue = 0;
		this.minSimilarityValue = 0;
		this.cutter = new ConstantCutter();
	}

	/**
	 * Costruisce una ranked list vuota consentendo di specificare l'algoritmo
	 * di taglio
	 * 
	 * @param pCutter
	 *        L'algoritmo di taglio della ranked list
	 */
	public BasicRankedList(ICutter pCutter) {
		this.rankedList = new TreeSet<RankedListElement>();
		this.maxSimilarityValue = 0;
		this.minSimilarityValue = 0;
		this.cutter = pCutter;
	}

	/**
	 * @see org.sesalab.IR.rankedList.IRankedList#addElement(org.sesalab.IR.rankedList.RankedListElement)
	 */
	@SuppressWarnings("unchecked")
	public boolean addElement(RankedListElement pElement) {
		if (pElement != null) {
			double similarity = pElement.getSimilarityValue();
			if (similarity < this.minSimilarityValue)
				this.minSimilarityValue = similarity;
			if (similarity > this.maxSimilarityValue)
				this.maxSimilarityValue = similarity;
			return this.rankedList.add(pElement);
		}
		return false;
	}

	/**
	 * @see org.sesalab.IR.rankedList.IRankedList#contains(org.sesalab.IR.rankedList.RankedListElement)
	 */
	public boolean contains(RankedListElement pElement) {
		return this.rankedList.contains(pElement);
	}

	/**
	 * @see org.sesalab.IR.rankedList.IRankedList#cut(double)
	 */
	public IRankedList cut(double pThreshold) {
		return this.cutter.cut(this, pThreshold);
	}

	/**
	 * @see org.sesalab.IR.rankedList.IRankedList#getMaxSimilarity()
	 */
	public double getMaxSimilarity() {
		return this.maxSimilarityValue;
	}

	/**
	 * @see org.sesalab.IR.rankedList.IRankedList#getMinSimilarity()
	 */
	public double getMinSimilarity() {
		return this.minSimilarityValue;
	}

	/**
	 * @see Iterable#iterator()
	 */
	public Iterator<RankedListElement> iterator() {
		Iterator<RankedListElement> documentListIterator = this.rankedList.iterator();
		return documentListIterator;
	}

	/**
	 * @see org.sesalab.IR.rankedList.IRankedList#removeElement(org.sesalab.IR.rankedList.RankedListElement)
	 */
	public boolean removeElement(RankedListElement pElement) {
		// TODO Aggiornare massimo e minimo
		if (pElement != null)
			return this.rankedList.remove(pElement);
		return false;
	}

	/**
	 * @see org.sesalab.IR.rankedList.IRankedList#setCutter(org.sesalab.IR.rankedList.cutter.ICutter)
	 */
	public void setCutter(ICutter pCutter) {
		this.cutter = pCutter;
	}

	/**
	 * @see IRankedList#size()
	 */
	public int size() {
		return this.rankedList.size();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		String str = "";
		int index = 1;
		Iterator<RankedListElement> documentListIterator = this.rankedList.iterator();
		while (documentListIterator.hasNext()) {
			str += index + ") " + documentListIterator.next() + "\n";
			index++;
		}
		return str;
	}

	// L'algoritmo di taglio della ranked list
	private ICutter cutter;

	// Massimo valore di somiglianza
	private double maxSimilarityValue;

	// Minimo valore di somiglianza
	private double minSimilarityValue;

	// La ranked list rappresentata come TreeSet
	private Collection<RankedListElement> rankedList;
}