/*
 *  IRankedList.java - SE@SA Lab 2006
 *  
 *  This file is part of COCONUT.
 *
 *  COCONUT is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  COCONUT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with COCONUT; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.sesalab.IR.rankedList;

import org.sesalab.IR.rankedList.cutter.ICutter;

/**
 * Definisce l'interfaccia di una ranked list
 * 
 * @author Paola Sgueglia
 */
public interface IRankedList extends Iterable<RankedListElement> {

	/**
	 * Consente di aggiungere un elemento alla ranked list
	 * 
	 * @param pElement
	 *        L'elemento da aggiungere
	 * @return <code>true</code> se l'inserimento  avvenuto con successo,
	 *         <code>false</code> altrimenti
	 */
	public boolean addElement(RankedListElement pElement);

	/**
	 * Consente di verificare se un elemento  presente all'interno della ranked
	 * list
	 * 
	 * @param pElement
	 *        L'elemento da ricercare
	 * @return <code>true</code> se l'elemento  presente, <code>false</code>
	 *         altrimenti
	 */
	public boolean contains(RankedListElement pElement);

	/**
	 * Consente di ottenere una ranked list tagliata ad un certo punto
	 * 
	 * @param pThreshold
	 *        Il punto di taglio
	 * @return La ranked list tagliata
	 */
	public IRankedList cut(double pThreshold);

	/**
	 * @return Il massimo valore di somiglianza
	 */
	public double getMaxSimilarity();

	/**
	 * @return Il minimo valore di somiglianza
	 */
	public double getMinSimilarity();

	/**
	 * Consente di rimuovere un elemento dalla ranked list
	 * 
	 * @param pElement
	 *        L'elemento da rimuovere
	 * @return <code>true</code> se l'eliminazione  avvenuta con successo,
	 *         <code>false</code> altrimenti
	 */
	public boolean removeElement(RankedListElement pElement);

	/**
	 * Consente di settare l'algoritmo per il taglio della ranked list
	 * 
	 * @param pCutter
	 *        L'algoritmo di taglio da settare
	 */
	public void setCutter(ICutter pCutter);

	/**
	 * Resituisce la dimensione della ranked list
	 * 
	 * @return La dimensione della ranked list
	 */
	public int size();
}