/*
 *  RankedListElement.java - SE@SA Lab 2006
 *  
 *  This file is part of COCONUT.
 *
 *  COCONUT is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  COCONUT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with COCONUT; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.sesalab.IR.rankedList;

import java.io.Serializable;

/**
 * Fornisce la rappresentazione di un elemento della ranked list
 * @param <ElementType> Tipo degli elementi source e target 
 * @author Paola Sgueglia
 */
public class RankedListElement<ElementType extends Comparable> implements Comparable, Serializable {

	private static final long serialVersionUID = -517625913121784241L;

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	@SuppressWarnings("unchecked")
	public int compareTo(Object toCompare){
		if (!(toCompare instanceof RankedListElement))
			throw new ClassCastException();

		if (this.equals(toCompare))
			return(0);
		
		RankedListElement elementToCompare = (RankedListElement) toCompare;
		
		// Si analizza il valore di somiglianza
		if (this.similarityValue.doubleValue() < elementToCompare.similarityValue.doubleValue())
			return (1);
		if (this.similarityValue.doubleValue() > elementToCompare.similarityValue.doubleValue())
			return (-1);

		// Si analizza il documento usato come query
		int comparison = this.source.compareTo(elementToCompare.source);
		if (comparison != 0)
			return (comparison);

		// Si analizza il documento recuperato
		comparison = this.target.compareTo(elementToCompare.target);
		if (comparison != 0)
			return (comparison);
		
		return 0;
	}

	/**
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object toCompare) {
		if (!(toCompare instanceof RankedListElement))
			throw new ClassCastException();
		RankedListElement elementToCompare = (RankedListElement) toCompare;
/*		boolean firstCondition = this.source.equals(elementToCompare.source) && this.target.equals(elementToCompare.target); 
		boolean secondCondition = this.source.equals(elementToCompare.target) && this.target.equals(elementToCompare.source); 
		return this.similarityValue.equals(elementToCompare.similarityValue) && (firstCondition || secondCondition);*/
		return this.similarityValue.equals(elementToCompare.similarityValue) && this.source.equals(elementToCompare.source) && this.target.equals(elementToCompare.target);
	}

	/**
	 * @return Il valore di somiglianza tra la query e il documento recuperato
	 */
	public Double getSimilarityValue() {
		return this.similarityValue;
	}

	/**
	 * @return Il documento usato come query
	 */
	public ElementType getSource() {
		return this.source;
	}

	/**
	 * @return Il documento recuperato dal tool
	 */
	public ElementType getTarget() {
		return this.target;
	}

	/**
	 * Setta il valore di somiglianza tra la query e il documento recuperato
	 * 
	 * @param pSimilarityValue
	 *        Valore di somiglianza tra la query e il documento recuperato
	 */
	public void setSimilarityValue(Double pSimilarityValue) {
		this.similarityValue = pSimilarityValue;
	}

	/**
	 * Setta il documento usato come query
	 * 
	 * @param pSource
	 *        Documento usato come query
	 */
	public void setSource(ElementType pSource) {
		this.source = pSource;
	}

	/**
	 * Setta il documento recuperato dal tool
	 * 
	 * @param pTarget
	 *        Documento recuperato dal tool
	 */
	public void setTarget(ElementType pTarget) {
		this.target = pTarget;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return this.source + "\t\t\t\t--> " + this.target + "\t\t\t\t : " + this.similarityValue;
	}

	// Valore di somiglianza tra la query e il documento recuperato
	private Double similarityValue;

	// Documento source 
	private ElementType source;

	// Documento target
	private ElementType target;
}
