/*
 *  VariableCutter.java - SE@SA Lab 2006
 *  
 *  This file is part of COCONUT.
 *
 *  COCONUT is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  COCONUT is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with COCONUT; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */package org.sesalab.IR.rankedList.cutter;

import java.util.Iterator;

import org.sesalab.IR.rankedList.BasicRankedList;
import org.sesalab.IR.rankedList.IRankedList;
import org.sesalab.IR.rankedList.RankedListElement;

/**
 * Fornisce l'implementazione dell'algoritmo di taglio basato su un valore variabile
 * 
 * @author Paola Sgueglia
 */
public class VariableCutter implements ICutter {
	/**
	 * @see org.sesalab.IR.rankedList.cutter.ICutter#cut(org.sesalab.IR.rankedList.IRankedList, double)
	 */
	public IRankedList cut(@SuppressWarnings("javadoc") IRankedList pRankedList, double pThreshold) {
		IRankedList cuttedRankedList = new BasicRankedList();
		if (pThreshold == ICutter.ALL)
			cuttedRankedList = pRankedList;
		else {
			double maxSimilarity = pRankedList.getMaxSimilarity();
			double minSimilarity = pRankedList.getMinSimilarity();
			double projectedThreshold = this.getProjectedThreshold(maxSimilarity, minSimilarity, pThreshold);
			
			Iterator<RankedListElement> rankedListIterator = pRankedList.iterator();
			while (rankedListIterator.hasNext()) {
				RankedListElement element = rankedListIterator.next();
				if (element.getSimilarityValue().doubleValue() >= projectedThreshold)
					cuttedRankedList.addElement(element);
				else
					break;
			}
		}
		return cuttedRankedList;
	}
	
	/**
	 * Calcola la proiezione del valore costante nell'intervall [maxSimilarity, minSimilarity]
	 * @param pMaxSimilarity
	 * 		  Il massimo valore di somiglianza 
	 * @param pMinSimilarity
	 * 		  Il minimo valore di somiglianza 
	 * @param pIntervalPercentage
	 * 		  La percentuale di intervallo da considerare 
	 * @return Il valore del threshold proiettato
	 */
	private double getProjectedThreshold(double pMaxSimilarity, double pMinSimilarity, double pIntervalPercentage) {
		double intervalSize = (pMaxSimilarity + 1) - (pMinSimilarity + 1);
		double projectedThreshold = ((intervalSize * pIntervalPercentage) / 100) + pMinSimilarity;
		return projectedThreshold;
	}
}