package traceability.recovery.coconut;

import traceability.links.Granularity;
import traceability.recovery.RecoveryTechniqueParams;

/**
 * Convenience interface used to help parameterize techniques that rely on the COCONUT IR API.
 * Techniques that extent COCONUTRecoveyTechnique must use this interface to implement their 
 * parameters.
 * 
 * @author Sam Klock
 *
 */
public abstract class COCONUTRecoveryTechniqueParams implements RecoveryTechniqueParams {
	
	private int threshold;
	private double minSim;
	private boolean useThreshold;
	private Granularity gran;
	
	/*
	 * (non-Javadoc)
	 * @see traceability.recovery.RecoveryTechniqueParams#setThreshold(int)
	 */
	@Override
	public void setThreshold(int threshold) {
		this.threshold = threshold;
	}
	
	/*
	 * (non-Javadoc)
	 * @see traceability.recovery.RecoveryTechniqueParams#setMinimumSimilarity(double)
	 */
	@Override
	public void setMinimumSimilarity(double similarity) {
		minSim = similarity;
	}
	
	/*
	 * (non-Javadoc)
	 * @see traceability.recovery.RecoveryTechniqueParams#setWhetherThreshold(boolean)
	 */
	@Override
	public void setWhetherThreshold(boolean whether) {
		useThreshold = whether;
	}
	
	/*
	 * (non-Javadoc)
	 * @see traceability.recovery.RecoveryTechniqueParams#setGranularity(traceability.links.Granularity)
	 */
	@Override
	public void setGranularity(Granularity gran) {
		this.gran = gran;
	}
	
	/**
	 * Retrieves the maximum number of traceability links to return.
	 * @return the absolute threshold
	 */
	public int getThreshold() {
		return threshold;
	}
	
	/**
	 * Retrieves the minimum similarity score of traceability links to return.
	 * @return the minimum similarity score
	 */
	public double getMinimumSimilarity() {
		return minSim;
	}
	
	/**
	 * Gets whether to use an absolute threshold or a minimum similarity as criteria for 
	 * returning results.
	 * @return if true, use the threshold value; else, use the similarity value
	 */
	public boolean getWhetherThreshold() {
		return useThreshold;
	}
	
	/**
	 * Retrieves the granularity setting for LSI.
	 * @return the granularity setting
	 */
	public Granularity getGranularity() {
		return gran;
	}
}
