package traceability.recovery.lsi;

import traceability.recovery.ParameterType;
import traceability.recovery.coconut.COCONUTRecoveryTechniqueParams;

/**
 * 
 * Recovery technique parameters for latent semantic indexing.
 * 
 * The sole technique-specific parameter supplied by this class is the subspace dimensionality 
 * for LSI to use, with a default value of 300.
 * 
 * @see traceability.recovery.lsi.LSIRecoveryTechnique
 * @author Sam Klock
 *
 */
public class LSIRecoveryTechniqueParams extends COCONUTRecoveryTechniqueParams {

	/**
	 * Parameter ID for the sole parameter used by LSI (subspace dimensionality).
	 */
	public static final String PARAMETER_ID = "DIM";
	private static final String PARAMETER_LABEL = "Subspace dimensionality";
	private static final ParameterType PARAMETER_TYPE = ParameterType.Integer;
	
	private int dimensionality = 300;
	
	/*
	 * (non-Javadoc)
	 * @see traceability.recovery.RecoveryTechniqueParams#getLabelForParameter(java.lang.String)
	 */
	@Override
	public String getLabelForParameter(String id) {
		if(PARAMETER_ID.equals(id)) {
			return PARAMETER_LABEL;
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see traceability.recovery.RecoveryTechniqueParams#getParameterIDs()
	 */
	@Override
	public String[] getParameterIDs() {
		String [] ret = { PARAMETER_ID };
		return ret;
	}

	/*
	 * (non-Javadoc)
	 * @see traceability.recovery.RecoveryTechniqueParams#getParameterValue(java.lang.String)
	 */
	@Override
	public Object getParameterValue(String id) {
		if(PARAMETER_ID.equals(id)) {
			return dimensionality;
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see traceability.recovery.RecoveryTechniqueParams#getTypeForParameter(java.lang.String)
	 */
	@Override
	public ParameterType getTypeForParameter(String id) {
		if(PARAMETER_ID.equals(id)) {
			return PARAMETER_TYPE;
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see traceability.recovery.RecoveryTechniqueParams#setValueForParameter(java.lang.String, java.lang.Object)
	 */
	@Override
	public void setValueForParameter(String id, Object val) {
		if(PARAMETER_ID.equals(id) && val instanceof Integer) {
			dimensionality = (Integer)val;
		}
	}
}
