package traceability.dialog;

import org.eclipse.core.resources.IProject;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * Dialog for obtaining a location for documentation for a particular project.
 * @author Sam Klock
 *
 */
public class DocumentationDialog extends Dialog {

	private static final int TEXT_WIDTH = 200;
	
	private Shell shell;
	private Text text;
	private String result;
	private IProject project;

	/**
	 * Constructor.
	 * @param parentShell parent shell for this dialog
	 * @param project project for which to specify a documentation location
	 */
	public DocumentationDialog(Shell parentShell, IProject project) {
		super(parentShell);
		this.project = project;
	}
	
	/**
	 * Shows this dialog.
	 * @return user-selected documentation location, or null if the user cancelled
	 */
	public Object open() {
		Shell parent = getParent();
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		setText("Select Documentation");
		shell.setLayout(new GridLayout(3, true));
		
		Label label = new Label(shell, 0);
		label.setText("Directory containing documentation: ");
		text = new Text(shell, SWT.SINGLE | SWT.BORDER);
		GridData data = new GridData();
		data.widthHint = TEXT_WIDTH;
		text.setLayoutData(data);
		Button button = new Button(shell, 0);
		button.setText("Browse...");
		button.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				return;
			}

			@Override
			public void widgetSelected(SelectionEvent e) {
				String filterPath = project.getLocation().makeAbsolute().toOSString();
				DirectoryDialog dirDialog = new DirectoryDialog(shell);
				dirDialog.setFilterPath(filterPath);
				text.setText(dirDialog.open());
			}
			
		});
		
		label = new Label(shell, 0);
		label.setText("");
		Button okay = new Button(shell, 0);
		okay.setText("OK");
		
		okay.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				return;				
			}

			@Override
			public void widgetSelected(SelectionEvent e) {
				result = text.getText();
				shell.close();
				
			}	
		});
		
		shell.pack();
		shell.open();
		Display display = parent.getDisplay();
		while(!shell.isDisposed()) {
			if(!display.readAndDispatch()) {
				display.sleep();
			}
		}
		return result;
	}
}
