package traceability.dialog;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Scanner;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import traceability.util.TraceabilitySelectionListener;

/**
 * Dialog for selecting a documentation artifact from a folder.
 * @author Sam Klock
 *
 */
public class SelectDocDialog extends Dialog {

	private static final int TEXT_WIDTH = 200;
	private static final int TEXT_HEIGHT = 500;
	
	private Shell shell;
	private Text contents;
	private List docs;
	private String selected;
	private String directory;
	
	/**
	 * Constructor.
	 * @param parentShell parent shell for the dialog
	 * @param directory directory from which to select documentation
	 */
	public SelectDocDialog(Shell parentShell, String directory) {
		super(parentShell);
		this.directory = directory;
	}
	
	/**
	 * Shows this dialog.
	 * @return user-selected documentation artifact, or null upon cancellation
	 */
	public Object open() {
		Shell parent = getParent();
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		setText("Select Documentation");
		shell.setLayout(new GridLayout(2, true));
		Label label = new Label(shell, 0);
		label.setText("Documents:");
		label = new Label(shell, 0);
		label.setText("Contents:");
		
		docs = new List(shell, SWT.BORDER | SWT.SINGLE);
		GridData data = new GridData();
		data.widthHint = TEXT_WIDTH;
		data.heightHint = TEXT_HEIGHT;
		docs.setLayoutData(data);
		contents = new Text(shell, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL | SWT.H_SCROLL);
		contents.setEditable(false);
		data = new GridData();
		data.widthHint = TEXT_WIDTH;
		data.heightHint = TEXT_HEIGHT;
		contents.setLayoutData(data);
		docs.addSelectionListener(new TraceabilitySelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				String selected = directory + File.separator + docs.getItem(docs.getSelectionIndex());
				
				Scanner scanner = null;
				try {
					scanner = new Scanner(new File(selected));
				} catch(FileNotFoundException ex) {
					contents.setText("(File not found)");
					return;
				}
				
				String text = "";
				while(scanner.hasNextLine()) {
					text += scanner.nextLine() + "\n";
				}
				contents.setText(text);
			}
			
		});
		fillDocList();
		
		if(docs.getItemCount() > 0) {
			docs.setSelection(0);
		}
		
		Button ok = new Button(shell, 0);
		ok.setText("OK");
		data = new GridData();
		data.horizontalSpan = 2;
		data.horizontalAlignment = SWT.CENTER;
		ok.setLayoutData(data);
		ok.addSelectionListener(new TraceabilitySelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				selected = directory + File.separator + docs.getItem(docs.getSelectionIndex());
				shell.close();
			}
			
		});
		
		shell.pack();
		shell.open();
		Display display = parent.getDisplay();
		while(!shell.isDisposed()) {
			if(!display.readAndDispatch()) {
				display.sleep();
			}
		}
		return selected;
	}
	
	/**
	 * Private helper method
	 */
	private void fillDocList() {
		File dir = new File(directory);
		if(dir.isDirectory() && dir.canRead()) {
			for(String s : dir.list()) {
				docs.add(s);
			}
		}
	}
}
