package traceability.popup.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
//import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import traceability.dialog.DocumentationDialog;
import traceability.dialog.SelectDocDialog;
import traceability.links.Granularity;
import traceability.management.Manager;
import traceability.management.ManagerException;
import traceability.util.NameUtilities;

/**
 * 
 * ObjectActionDelegate for adding traceability links for a selected artifact.
 * 
 * @author Sam Klock
 *
 */
public class AddLink implements IObjectActionDelegate {

	/**
	 * Used for child dialogs.
	 */
	private Shell shell;
	/**
	 * Specifies artifact for which to add a link.
	 */
	private Object target;
	
	/**
	 * Constructor.
	 */
	public AddLink() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction, org.eclipse.ui.IWorkbenchPart)
	 */
	@Override
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		shell = targetPart.getSite().getShell();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	@Override
	public void run(IAction action) {
		Manager theManager = Manager.getInstance();
		
		//First: need to set the project
		try {
			if(target instanceof IMethod) {
				IMethod method = (IMethod)target;
				theManager.setProject(method.getJavaProject().getProject());
			} else if(target instanceof IType) {
				IType theClass = (IType)target;
				theManager.setProject(theClass.getJavaProject().getProject());
			} else if(target instanceof IPackageFragment) {
				IPackageFragment thePackage = (IPackageFragment)target;
				theManager.setProject(thePackage.getJavaProject().getProject());
			} else if(target instanceof IFile) {
				IFile theFile = (IFile)target;
				theManager.setProject(theFile.getProject());
			}
		} catch(ManagerException e) {	//TODO improve exception handling
			e.printStackTrace();
		} catch(Exception e) {
			e.printStackTrace();
		} catch(Error e) {
			e.printStackTrace();
		}
		
		//Next: set the documentation location
		String docLoc = theManager.getDocumentationLocation();
		if(docLoc == null) {
			docLoc = (String)(new DocumentationDialog(shell, theManager.getProject())).open();
			if(docLoc == null) {	//user canceled
				return;
			}
			try {
				theManager.setDocumentationLocation(docLoc);
			} catch(Exception e) {
				e.printStackTrace();
			}
		}
		
		//Next: get documentation to which to trace
		String doc = (String)(new SelectDocDialog(shell, docLoc)).open();
		if(doc == null) {	//user canceled
			return;
		}
		
		//Finally: add the link
		try {
			if(target instanceof IMethod) {
				IMethod method = (IMethod)target;
				theManager.addLink(NameUtilities.getNameForJavaMethod(method), doc, Granularity.Method);
			} else if(target instanceof IType) {
				IType theClass = (IType)target;
				theManager.addLink(NameUtilities.getNameForJavaClass(theClass), doc, Granularity.Class);
			} else if(target instanceof IPackageFragment) {
				IPackageFragment thePackage = (IPackageFragment)target;
				theManager.addLink(NameUtilities.getNameForJavaPackage(thePackage), doc, Granularity.Package);
			} else if(target instanceof IFile) {
				IFile theFile = (IFile)target;
				theManager.addLink(theFile.getProjectRelativePath().toPortableString(), doc, Granularity.File);
			} else {
				//TODO handle this case
			}
			theManager.saveSet();
		} catch(ManagerException e) {
			e.printStackTrace();
		} catch(Exception e) {
			e.printStackTrace();
		} catch(Error e) {
			e.printStackTrace();
		}
		//TODO improve exception handling
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void selectionChanged(IAction action, ISelection selection) {
		//Only handle structured selections
		if(selection.isEmpty() || 
				!(selection instanceof IStructuredSelection)) {
			return;
		}
		
		//Only accept first selection
		target = ((IStructuredSelection)selection).getFirstElement();
	}

}
