package traceability.popup.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import traceability.management.Manager;
import traceability.management.ManagerException;
import traceability.util.NameUtilities;
import traceability.views.TraceabilityContentProvider;
import traceability.views.TraceabilityView;

/**
 * 
 * ObjectActionDelegate for viewing traceability links for a given Java source artifact.
 * 
 * @author Sam Klock
 *
 */
public class ViewLinks implements IObjectActionDelegate {

	/**
	 * Target artifact for which to view links.
	 */
	private Object target;
	
	/**
	 * Constructor.
	 */
	public ViewLinks() {
		super();
	}

	/*
	 * This method doesn't do anything, but is required by IObjectActionDelegate.
	 * 
	 * (non-Javadoc)
	 * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(org.eclipse.jface.action.IAction, org.eclipse.ui.IWorkbenchPart)
	 */
	@Override
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		//Nothing
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
	 */
	@Override
	public void run(IAction action) {
		TraceabilityView view = TraceabilityView.getInstance();
		if(view == null) {
			return;
		}
		
		Manager theManager = Manager.getInstance();
		try {
			String input = TraceabilityContentProvider.FOR_SRC;
			if(target instanceof IMethod) {
				IMethod method = (IMethod)target;
				theManager.setProject(method.getJavaProject().getProject());
				input += NameUtilities.getNameForJavaMethod(method);
			} else if(target instanceof IType) {
				IType theClass = (IType)target;
				theManager.setProject(theClass.getJavaProject().getProject());
				input += NameUtilities.getNameForJavaClass(theClass); 
			} else if(target instanceof IPackageFragment) {
				IPackageFragment thePackage = (IPackageFragment)target;
				theManager.setProject(thePackage.getJavaProject().getProject());
				input += NameUtilities.getNameForJavaPackage(thePackage);
			} else if(target instanceof IFile) {
				IFile theFile = (IFile)target;
				theManager.setProject(theFile.getProject());
				input += theFile.getProjectRelativePath().toPortableString();
			} else if(target instanceof IProject) {
				theManager.setProject((IProject)target);
				input = TraceabilityContentProvider.FOR_PROJECT;
			}
			view.setInput(input);
		} catch(ManagerException e) {
			e.printStackTrace();
		} catch(Exception e) {
			e.printStackTrace();
		} catch(Error e) {
			e.printStackTrace();
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.IActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void selectionChanged(IAction action, ISelection selection) {
		//Only handle structured selections
		if(selection.isEmpty() || 
				!(selection instanceof IStructuredSelection)) {
			return;
		}
		
		//Only accept first selection
		target = ((IStructuredSelection)selection).getFirstElement();
	}

}
