package traceability.recovery;

import java.util.Set;

import org.eclipse.core.resources.IProject;

import traceability.links.TraceabilityLink;

/**
 * 
 * Interface for traceability links recovery techniques appied via the Traceability Links Recovery wizard.
 * 
 * @author Sam Klock
 *
 */
public interface RecoveryTechnique {
	
	/**
	 * Retrieves the name of this technique.
	 * @return the technique's name
	 */
	public String getName();
	
	/**
	 * Retrieves the parameters for this technique.
	 * @return RecoveryTechniqueParams specified for this technique
	 */
	public RecoveryTechniqueParams getParameters();
	
	/**
	 * Runs this technique on the selected project.
	 * @param project the project for which to recover links
	 * @return true if successful; false if an error occurred
	 */
	public boolean run(IProject project);
	
	/**
	 * Retrieves the traceability links recovered by this technique.
	 * @return a (possibly empty) set of links
	 */
	public Set<TraceabilityLink> getRecoveredLinks();
}
