package traceability.recovery;

import traceability.links.Granularity;

/**
 * 
 * Interface for specifying parameters passed from the Traceability Links Recovery wizard to 
 * a recovery technique.
 * 
 * @author Sam Klock
 *
 */
public interface RecoveryTechniqueParams {
	/**
	 * Retrieves string IDs for the technique's parameters.
	 * @return array of string IDs
	 */
	public String [] getParameterIDs();
	
	/**
	 * Retrieves a textual label for the parameter with the given ID.
	 * @param id ID for the parameter
	 * @return textual label for the parameter
	 */
	public String getLabelForParameter(String id);
	
	/**
	 * Retrieves a type for the selected parameter.
	 * @param id ID for the parameter
	 * @return type for the parameter
	 */
	public ParameterType getTypeForParameter(String id);
	
	/**
	 * Sets the value of the given parameter.
	 * 
	 * It is up to the technique to decide what to do if the value is illegal.
	 * 
	 * @param id ID for the parameter
	 * @param val value to give for the parameter
	 */
	public void setValueForParameter(String id, Object val);
	
	/**
	 * Retrieves the value of the given parameter.
	 * @param id ID for the parameter
	 * @return value of the parameter
	 */
	public Object getParameterValue(String id);
	
	/**
	 * Sets the granularity for this technique.
	 * @param granularity the granularity
	 */
	public void setGranularity(Granularity granularity);
	
	/**
	 * Sets the maximum number of traceability links to collect, if applicable.
	 * @param threshold maximum number
	 */
	public void setThreshold(int threshold);
	
	/**
	 * Sets the minimum similarity of traceability links to collect, if applicable.
	 * @param min minimum similarity
	 */
	public void setMinimumSimilarity(double min);
	
	/**
	 * Sets whether to use the threshold or minimum similarity values when determining 
	 * what links to return.
	 * @param whether if true, use the threshold; if false, use the minimum similarity values
	 */
	public void setWhetherThreshold(boolean whether);
}
