package traceability.views;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Link;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.ide.IDE;

import traceability.links.TraceabilityLink;
import traceability.management.Manager;
import traceability.util.NameUtilities;
import traceability.util.TraceabilitySelectionListener;

/**
 * 
 * Selection listener handling clicks on links to source code artifacts.
 * 
 * This listener currently assumes that all source artifacts are Java artifacts.
 * 
 * @author Sam Klock
 *
 */
public class SourceLinkListener extends TraceabilitySelectionListener {
	
	/**
	 * Single instance
	 */
	private static SourceLinkListener instance = new SourceLinkListener();
	
	/**
	 * Viewer to which this listener is attached.
	 */
	private static TraceabilityView viewer;
	
	/**
	 * Constructor.  Not publicly instantiable.
	 */
	private SourceLinkListener() {
		
	}
	
	/**
	 * Retrieves the single instance of the listener.
	 * @return the listener
	 */
	public static SourceLinkListener getInstance() {
		if(viewer == null) {
			viewer = TraceabilityView.getInstance();
		}
		return instance;
	}

	/*
	 * (non-Javadoc)
	 * @see traceability.util.TraceabilitySelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	@Override
	public void widgetSelected(SelectionEvent e) {
		Link source = (Link)e.getSource();
		TraceabilityLink link = (TraceabilityLink)source.getData();
		String target = link.getSource();
		
		IWorkbenchPage page = viewer.getSite().getPage(); 
		
		Manager manager = Manager.getInstance();
		IProject project = manager.getProject();
		IFile file = null;
		IPath path = null;
		
		switch(link.getSourceGranularity()) {
		case File: 
			IPath location = Path.fromPortableString(target);
			location = location.makeRelativeTo(project.getLocation());
			file = project.getFile(location);
			break;
		case Class:
			//TODO would be nice to set the editor to open at the right line
			//this is complicated to do
			IType theClass = NameUtilities.getJavaClassFor(target);
			path = theClass.getPath();
			path = path.makeRelativeTo(project.getLocation());
			file = project.getFile(path);
			break;
		case Method:
			//TODO see above
			IMethod theMethod = NameUtilities.getJavaMethodFor(target);
			path = theMethod.getPath();
			path = path.makeRelativeTo(project.getLocation());
			file = project.getFile(path);
			break;
		case Package:
			//Don't bother with packages
			//TODO Bother at some point perhaps...
			return;
		}
		
		/* Open the file */
		try {
			IDE.openEditor(page, file);
		} catch (PartInitException exc) {
			MessageDialog.openError(viewer.getViewer().getTable().getShell(), 
					"Error Opening File", 
					"Could not open file for editing.");
		} catch (Exception exc) {
			exc.printStackTrace();
		} catch (Error err) {
			err.printStackTrace();
		}
	}
}
