package traceability.wizard;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import traceability.management.Manager;
import traceability.util.TraceabilitySelectionListener;

/**
 * Wizard page for selecting a folder with documentation.
 * 
 * @author Sam Klock
 *
 */
public class DocumentationWizardPage extends WizardPage {

	/**
	 * Default text box width.
	 */
	private static final int TEXT_WIDTH = 200;
	
	/**
	 * Text box containing path to documentation.
	 */
	private Text documentationText;
	
	/**
	 * Constructor.
	 */
	public DocumentationWizardPage() {
		super("Select Folder with Documentation");
		setDescription("Select the folder containing the documentation to which to trace.");
	}
	
	/**
	 * Retrieves the user-selected directory containing the desired documentation.
	 * @return documentation directory
	 */
	public String getDocumentationLocation() {
		return documentationText.getText();
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayout(new GridLayout(3, true));
		
		/* Label */
		Label label = new Label(composite, SWT.NONE);
		label.setText("Folder with documentation:");
		
		/* Text box */
		documentationText = new Text(composite, SWT.BORDER);
		GridData bounds = new GridData();
		bounds.widthHint = TEXT_WIDTH;
		documentationText.setLayoutData(bounds);
		Manager manager = Manager.getInstance();
		if(manager.getDocumentationLocation() != null) {
			documentationText.setText(manager.getDocumentationLocation());
		}
		
		/* Browse button */
		Button browse = new Button(composite, SWT.PUSH);
		browse.setText("Browse...");
		browse.addSelectionListener(new TraceabilitySelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				DirectoryDialog dirDialog = new DirectoryDialog(getShell());
				Manager manager = Manager.getInstance();
				IProject project = manager.getProject();
				if(!project.isOpen()) {
					try {
						project.open(null);
					} catch (CoreException e1) {
						MessageDialog.openError(getShell(), "Error Opening Project", 
								"An error occurred while opening the current project: " + 
								e1.getMessage());
						String text = dirDialog.open();
						if(text != null) {
							documentationText.setText(text);
						}
						return;
					}
					
				}
				String filterPath = project.getLocation().makeAbsolute().toOSString();
				dirDialog.setFilterPath(filterPath);
				String text = dirDialog.open();
				if(text != null) {
					documentationText.setText(text);
				}
			}
		});
		
		setControl(composite);
	}

}
