package traceability.wizard;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import traceability.links.Granularity;
import traceability.util.TraceabilitySelectionListener;

/**
 * 
 * Wizard page for selecting source artifact granularity.
 * 
 * @author Sam Klock
 *
 */
public class GranularityWizardPage extends WizardPage {

	/**
	 * Constructor.
	 */
	public GranularityWizardPage() {
		super("Select Granularity");
		setDescription("Select a level of granularity with which to view source artifacts.");
	}
	
	/**
	 * The user-selected granularity.
	 */
	private Granularity selectedGranularity = Granularity.Package;
	
	/**
	 * Selection listener for radio buttons on this page.
	 */
	private TraceabilitySelectionListener radioListener = new TraceabilitySelectionListener() {

		@Override
		public void widgetSelected(SelectionEvent e) {
			Button source = (Button)e.getSource();
			Granularity granularity = (Granularity)source.getData();
			selectedGranularity = granularity;
		}
		
	};
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		composite.setLayout(layout);
		
		Button packageButton = new Button(composite, SWT.RADIO);
		packageButton.setData(Granularity.Package);
		packageButton.setText("Package");
		packageButton.setSelection(true);
		packageButton.addSelectionListener(radioListener);
		
		Button classButton = new Button(composite, SWT.RADIO);
		classButton.setData(Granularity.Class);
		classButton.setText("Class");
		classButton.setSelection(false);
		classButton.addSelectionListener(radioListener);
		
		Button methodButton = new Button(composite, SWT.RADIO);
		methodButton.setData(Granularity.Method);
		methodButton.setText("Method");
		methodButton.setSelection(false);
		methodButton.addSelectionListener(radioListener);
		
		Button fileButton = new Button(composite, SWT.RADIO);
		fileButton.setData(Granularity.File);
		fileButton.setText("File");
		fileButton.setSelection(false);
		fileButton.addSelectionListener(radioListener);
		
		setControl(composite);
	}

	/**
	 * Retrieves the user-selected granularity.
	 * @return the granularity
	 */
	public Granularity getSelectedGranularity() {
		return selectedGranularity;
	}
}
